from django.contrib.auth import get_user_model
from rest_framework import serializers

from src import helper

User = get_user_model()


class DynamicFieldsModelSerializer(serializers.ModelSerializer):
    """
    A ModelSerializer that takes an additional `fields` argument that
    controls which fields should be displayed.
    """

    id = serializers.CharField(source="uuid", read_only=True)

    def __init__(self, *args, **kwargs):
        # Don't pass the 'fields' arg up to the superclass
        fields = kwargs.pop("fields", None)
        exclude = kwargs.pop("exclude", None)

        # Adding this next line to the documented example
        read_only_fields = kwargs.pop("read_only_fields", None)

        # Instantiate the superclass normally
        super(DynamicFieldsModelSerializer, self).__init__(*args, **kwargs)

        if fields is not None:
            # Drop any fields that are not specified in the `fields` argument.
            allowed = set(fields)
            existing = set(self.fields)
            for field_name in existing - allowed:
                self.fields.pop(field_name)

        if exclude is not None:
            existing = set(self.fields)
            for field_name in existing:
                if field_name in exclude:
                    self.fields.pop(field_name)

        # another bit we're adding to documented example, to take care of readonly fields
        if read_only_fields is not None:
            for f in read_only_fields:
                try:
                    self.fields[f].read_only = True
                except KeyError:
                    # not in fields anyway
                    pass

        for field_name, field in self.fields.items():
            if isinstance(field, serializers.ManyRelatedField):
                related_model = field.child_relation.queryset.model
                slug_related_field = serializers.SlugRelatedField(
                    slug_field="uuid",
                    required=False,
                    allow_null=True,
                    allow_empty=True,
                    queryset=related_model.objects.all(),
                    many=True,
                )
                self.fields[field_name] = slug_related_field
            elif isinstance(field, serializers.PrimaryKeyRelatedField):
                related_model = field.queryset.model
                slug_related_field = serializers.SlugRelatedField(
                    slug_field="uuid",
                    required=False,
                    allow_null=True,
                    allow_empty=True,
                    queryset=related_model.objects.all(),
                )
                self.fields[field_name] = slug_related_field

    @staticmethod
    def remove_null_key_value_pair(dictionary):
        return helper.remove_null_key_value_pair(dictionary)

    @staticmethod
    def validate_float_value(value):
        try:
            return float(value)
        except ValueError:
            raise serializers.ValidationError("A valid number is required.")

    def unique_value_validator(
        self,
        field,
        value,
        error_message,
        look_up,
        check_parent_model=False,
        model=None,
        **kwargs,
    ):
        error_message = serializers.ValidationError(error_message)
        instance = self.instance
        model = self.Meta.model if not model else model
        if check_parent_model:
            model = model._meta.get_parent_list()[0]

        kwargs[look_up] = value

        if instance:
            if (
                getattr(instance, field) != value
                and model.objects.filter(**kwargs).exists()
            ):
                raise error_message
        elif model.objects.filter(**kwargs).exists():
            raise error_message
        return value


class ReadOnlyModelSerializer(DynamicFieldsModelSerializer):
    def get_fields(self):
        fields = super().get_fields()
        for field in fields.values():
            # set all field to read_only which eventually speed the response
            field.read_only = True
        return fields



def get_response_message(data, model, action="create"):
    obj_response_message = data.get("response_message", None)
    response_message = f"{obj_response_message} {action}d successfully"
    return response_message


def remove_null_key_value_pair(dictionary):
    return {k: v for k, v in dictionary.items() if v not in [None, "", "N/A"]}




from django.http import JsonResponse
from rest_framework import renderers, status

RESPONSE_MESSAGE = {
    status.HTTP_200_OK: "Data",
    status.HTTP_201_CREATED: "Created",
    status.HTTP_204_NO_CONTENT: "No Content",
    status.HTTP_400_BAD_REQUEST: "Bad Request",
    status.HTTP_401_UNAUTHORIZED: "Unauthorized",
    status.HTTP_403_FORBIDDEN: "Forbidden",
    status.HTTP_405_METHOD_NOT_ALLOWED: "Method Not Found",
    status.HTTP_404_NOT_FOUND: "Not Found",
    status.HTTP_500_INTERNAL_SERVER_ERROR: "Internal Server Error",
    status.HTTP_501_NOT_IMPLEMENTED: "Method not Implemented",
}


class CustomRenderer(renderers.JSONRenderer):
    def render(self, data, accepted_media_type=None, renderer_context=None):
        status_code = renderer_context["response"].status_code

        api_response_message = None

        if isinstance(data, dict):
            api_response_message = data.pop("message", None)

        if data and "response_data" in data:
            data = data.pop("response_data", None)
            data.pop("response_message", None)

        if status_code in [
            status.HTTP_200_OK,
            status.HTTP_201_CREATED,
            status.HTTP_204_NO_CONTENT,
        ]:
            response = {
                "success": True,
                "message": api_response_message,
                "status": status_code,
            }
            if data and "additional_info" in data:
                response["additional_info"] = data.get("additional_info")

            if data is not None:
                if "results" in data:
                    response.update(
                        {
                            "count": data["count"],
                            "next": data["next"],
                            "previous": data["previous"],
                            "results": data["results"],
                        }
                    )
                else:
                    response.update({"results": data})
        else:
            response = {
                "success": False,
                "error": data,  # Remove the extra "error" key here
                "message": api_response_message,
                "status": status_code,
            }
            if "detail" in data:
                response["error"]["non_field_errors"] = [data["detail"]]
            elif "non_field_errors" in data:
                response["error"]["non_field_errors"] = data["non_field_errors"]

        return JsonResponse(data=response)



"""
pytz setup script
"""
import os
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

me = 'Aman Bothra'
memail = 'amanbothra1777@gmail.com'

packages = ['dj_rest_kit']
package_dir = {'dj_rest_kit': '/src'}
install_requires = open('requirements.txt', 'r').readlines()

setup(
    name='dj_rest_kit',
    version=re.sub(r'^v', '', os.environ.get('CI_COMMIT_TAG'), flags=re.IGNORECASE),
    zip_safe=True,
    description='A few extensions to DRF.',
    author=me,
    author_email=memail,
    maintainer=me,
    maintainer_email=memail,
    install_requires=install_requires,
    url='https://github.com/prashantgabu-e/dj-rest-kit',
    license=open('LICENSE', 'r').read(),
    keywords=['djangorestframework', 'drf'],
    packages=packages,
    package_dir=package_dir,
    platforms=['Independant'],
    classifiers=[
        'Development Status :: 1 - beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters, status, viewsets
from rest_framework.response import Response
from rest_framework.views import APIView

from src import mixins
from src.helper import get_response_message
from src.renderer import CustomRenderer


class BaseAPIView(APIView):
    renderer_classes = [CustomRenderer]


class BaseGenericViewSet(mixins.ViewSetMixin, viewsets.GenericViewSet):
    filter_backends = (filters.SearchFilter, DjangoFilterBackend)

    model = None
    filterset_class = None

    class Meta:
        abstract = True

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context["user"] = self.request.user
        return context


class BaseViewSet(mixins.ViewSetMixin, viewsets.ModelViewSet):
    filter_backends = (filters.SearchFilter, DjangoFilterBackend)
    http_method_names = ["get", "post", "patch", "delete", "put"]

    model = None
    filterset_class = None

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context["user"] = self.request.user
        return context

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        response_data = serializer.data
        response = {
            "response_data": response_data,
            "message": get_response_message(response_data, self.model, self.action),
        }
        return Response(data=response, status=status.HTTP_201_CREATED, headers=headers)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, "_prefetched_objects_cache", None):
            instance._prefetched_objects_cache = {}
        response_data = serializer.data
        action_name = self.action
        if partial:
            action_name = "update"
        response = {
            "response_data": response_data,
            "message": get_response_message(response_data, self.model, action_name),
        }
        return Response(data=response, status=status.HTTP_200_OK)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.delete()
        return Response({'message': 'Deleted successfully'})



from django.contrib.postgres.fields.array import ArrayField
from django.db.models import CharField
from django.utils.encoding import force_str
from rest_framework import permissions, status
from rest_framework.response import Response


class ViewSetMixin:
    permission_classes = (permissions.IsAuthenticated,)

    def perform_update(self, serializer):
        serializer.save(updated_by=self.request.user.name)

    @staticmethod
    def custom_error_response(
            error_message, response_status=status.HTTP_400_BAD_REQUEST
    ):
        if isinstance(error_message, dict):
            error_response = error_message
        elif isinstance(error_message, list):
            error_response = {"non_field_errors": error_message}
        else:
            error_response = {"non_field_errors": [error_message]}
        return Response(error_response, status=response_status)

    @classmethod
    def get_choices_for_model_fields(cls, model):
        parsed_choices = {}
        for field in model._meta.get_fields():
            if (
                    isinstance(field, CharField)
                    and field.choices
                    or isinstance(field, ArrayField)
            ):
                field_choices = field.choices
                if isinstance(field, ArrayField) and isinstance(
                        field.base_field, CharField
                ):
                    field_choices = field.base_field.choices
                parsed_choices[field.name] = [
                    {
                        "value": choice_value,
                        "display_name": force_str(choice_name, strings_only=True),
                    }
                    for choice_value, choice_name in dict(field_choices).items()
                ]
        return parsed_choices



import django_filters
from django_filters import BaseInFilter, NumberFilter
from django_filters import rest_framework as filters
from django_filters.filters import OrderingFilter


class BaseFilter(filters.FilterSet):
    order_by_field = "ordering"


class BaseOrderingFilter(OrderingFilter):
    pass


class BaseNumberInFilter(BaseInFilter, NumberFilter):
    pass



from rest_framework import pagination


class CustomPagination(pagination.LimitOffsetPagination):
    default_limit = 10

    def paginate_queryset(self, queryset, request, view=None):
        self.count = self.get_count(queryset)
        self.limit = self.get_limit(request, total_count=self.count)
        if self.limit is None:
            return None

        self.offset = self.get_offset(request)
        self.request = request
        if self.count > self.limit and self.template is not None:
            self.display_page_controls = True

        if self.count == 0 or self.offset > self.count:
            return []
        return list(queryset[self.offset: self.offset + self.limit])

    def get_limit(self, request, total_count=None):
        if request.query_params.get(self.limit_query_param) == "all":
            return total_count
        return super().get_limit(request=request)
